# Copyright (C) 2010-2012 Team Gaspacho (see README for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from os.path import join, isdir
from os import makedirs
from shutil import rmtree

from gaspacho import get_choices
from gaspacho.group import get_all_groups
from gaspacho.config import store_dir_apply, default_separator
from gaspacho import get_softwares, get_oses
from gaspacho.conflevel import get_confcomputer, get_confuser, get_confcontext
from gaspacho.log import trace #, logger

try:
    import json
except:
    #for python 2.5
    import simplejson as json

@trace
def construct_ret(ret, level, group, osname, osversion, usertype, username,
        extension, path, key, tvariable):
    ret[group][osname][osversion][usertype][username].setdefault(level, {})
    ret[group][osname][osversion][usertype][username][level].setdefault(
                extension, {})
    ret[group][osname][osversion][usertype][username][level][extension].setdefault(
                path, {})
    ret[group][osname][osversion][usertype][username][level][extension][path][key] = tvariable

@trace
def construct_dict_group(ret, group, user):
    if user == None:
        username = None
        usertype = None
    elif user == 'ALL':
        username = 'ALL'
        usertype = ''
    else:
        username = user.name
        usertype = user.type
    
    if group.softs != []:
        softwares = group.softs
    else:
        softwares = get_softwares()
    if group.oses != []:
        oses = group.oses
    else:
        oses = get_oses()
    for Os in oses:
        osname = Os.name
        if not ret[group].has_key(osname):
            ret[group][osname] = {}
        for osversion in Os.get_versions():
            osver = osversion.name
            if osver == u'ALL':
                continue
            if not ret[group][osname].has_key(osver):
                ret[group][osname][osver] = {}
            if not ret[group][osname][osver].has_key(usertype):
                ret[group][osname][osver][usertype] = {}
            if not ret[group][osname][osver][usertype].has_key(username):
                ret[group][osname][osver][usertype][username] = {}

@trace
def get_choices_by_group(ret, group, user):
    confcomputer = get_confcomputer()
    confuser = get_confuser()
    confcontext = get_confcontext()
    if user == None:
        username = None
        usertype = None
    else:
        username = user.name
        usertype = user.type
    rules_for_user = []
    rules_for_context = []
    for rule, choice in get_choices(group, user).items():
        choice, hchoice = choice
        if choice == None:
            choice = hchoice
        if choice == None:
            raise Exception('choice must not be None')
        tchoice = choice.state
        tvalue = choice.value
        rtype = rule.type
        conf = rule.conflevels
        if rtype == u'multi':
            mseparator = rule.options.get(u'separator', default_separator)
            tmvalue = tvalue.split(mseparator)
            i = 0
            mvalue = {}
            for val in rule.options[u'values']:
                mvalue[val[u'name']] = tmvalue[i]
                i+=1
        for variable in rule.vars:
            von = variable.value_on
            voff = variable.value_off
            key = variable.name
            typ = variable.type
            if tchoice == u'on':
                if von == None:
                    value = tvalue
                else:
                    if rtype == u'multi':
                        tvon = []
                        for v in von.split(mseparator):
                            tvon.append(mvalue.get(v, v))
                        tvon = unicode(mseparator.join(tvon))
                        value = tvon
                    else:
                        value = von
            elif tchoice == u'off':
                value = voff
            elif tchoice == u'free':
                value = ''
            else:
                raise Exception('unknown choice: %s'%tchoice)
            if value == u'IGNORE':
                #if value == IGNORE, rule must not be changed
                continue
            for platform in variable.plats:
                osname = platform.osversion.os.name
                osversion = platform.osversion.name
                path = platform.path.name
                extension = platform.path.extension
                info = platform.path.info
                if osversion == u'ALL':
                    osversions = []
                    for osversion in platform.osversion.os.get_versions():
                        osversion = osversion.name
                        if osversion == u'ALL':
                            continue
                        osversions.append(osversion)
                else:
                    osversions = [osversion]
                for osversion in osversions:
                    tvariable = {'key': key,
                                 'type': typ,
                                 'value': value}
                    if info != None:
                        tvariable['info'] = info
                    if user == None:
                        if confcomputer in conf:
                            construct_ret(ret, 'computer', group, osname,
                                    osversion, usertype, username, extension,
                                    path, key, tvariable)
                        if confuser in conf:
                            rules_for_user.append((osname, osversion, extension,
                                    path, key, tvariable))
                        if confcontext in conf:
                            rules_for_context.append((osname, osversion,
                                    extension, path, key, tvariable))
                    else:
                        if tchoice == u'free':
                            try:
                                del(ret[leve][group][osname][osversion][usertype][username][extension][path][key])
                            except:
                                pass
                        else:
                            if conf == confcomputer:
                                raise Exception('User must not have confcomputer')
                            elif conf == confuser:
                                construct_ret(ret, 'user', group, osname,
                                        osversion, usertype, username,
                                        extension, path, key, tvariable)
                            else:
                                construct_ret(ret, 'context', group, osname,
                                        osversion, usertype, username,
                                        extension, path, key, tvariable)
    if user == None:
        for user in group.users:
            construct_dict_group(ret, group, user)
            get_choices_by_group(ret, group, user)
        #FIXME: should not be ALL
        construct_dict_group(ret, group, 'ALL')
        for osname, osversion, extension, path, key, tvariable in \
                    rules_for_user:
            construct_ret(ret, 'user', group, osname, osversion, '', 'ALL',
                    extension, path, key, tvariable)
        for osname, osversion, extension, path, key, tvariable in \
                    rules_for_context:
            construct_ret(ret, 'context', group, osname, osversion, '', 'ALL',
                    extension, path, key, tvariable)

@trace
def apply_choices():
    ret = {}
    groups = list(get_all_groups())
    for group in groups:
        if ret.has_key(group):
            raise Exception('group already exists')
        ret[group] = {}
        construct_dict_group(ret, group, None)
        get_choices_by_group(ret, group, None)
    ret_tree = []
    rmtree(store_dir_apply)
    makedirs(store_dir_apply)
    for group in groups:
        osnames = ret[group]
        cret = {'user': {}, 'usergroup': {}, 'all': {}}
        tret = {'computer': {}, 'user': cret, 'context': cret}
        filedir = join(store_dir_apply, group.name)
        if isdir(filedir):
            raise Exception('%s already exists'%filedir)
        makedirs(filedir)
        for osname, osversions in osnames.items():
            for osversion, usertypes in osversions.items():
                for usertype, users in usertypes.items():
                    for user, levels in users.items():
                        for level, extensions in levels.items():
                            retjson = {}
                            tosname = '%s_%s' % (osname, osversion)
                            for extension, paths in extensions.items():
                                retjson[extension] = []
                                for path, variables in paths.items():
                                    tvar = []
                                    for name, variable in variables.items():
                                        tvar.append(variable)
                                    if tvar != []:
                                        retjson[extension].append({'path': path,
                                                        'variables': tvar})
                            if level == 'computer':
                                filedir = join(store_dir_apply, group.name,
                                        osname, osversion)
                                filename = join(filedir, 'computer.gasp')
                                tret['computer'][tosname] = filename
                            else:
                                filedir = join(store_dir_apply, group.name,
                                        osname, osversion, usertype, level)
                                filename = join(filedir, '%s.gasp'%user)
                                if usertype != '':
                                    if not tret[level][usertype].has_key(user):
                                        tret[level][usertype][user] = {}
                                    tret[level][usertype][user][tosname] = \
                                            filename
                                else:
                                    tret[level]['all'][tosname] = filename

                            if not isdir(filedir):
                                makedirs(filedir)
                            fh = file(filename, 'w')
                            json.dump(retjson, fh, indent=1)
                            fh.close()
        for name, typ in group.get_computers():
            ret_tree.insert(0, {'group': (name, typ), 'path': tret})
    filename = join(store_dir_apply, 'gaspacho.json')
    fh = file(filename, 'w')
    json.dump(ret_tree, fh, indent=1)
    fh.close()

# vim: ts=4 sw=4 expandtab
